/**
 * Interceptor
 * Belongs to Decentraleyes.
 *
 * @author      Thomas Rientjes
 * @since       2016-04-06
 * @license     MPL 2.0
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 */

'use strict';

/**
 * Interceptor
 */

var interceptor = {};

/**
 * Public Methods
 */

interceptor.handleRequest = function (requestDetails, tabIdentifier, tab) {

    let validCandidate, tabDomain, targetDetails, targetPath;

    validCandidate = requestAnalyzer.isValidCandidate(requestDetails, tab);

    if (!validCandidate) {

        return {
            'cancel': false
        };
    }

    tabDomain = helpers.extractDomainFromUrl(tab.url, true);

    if (tabDomain === null) {
        tabDomain = Address.EXAMPLE;
    }

    if (requestDetails.type === WebRequestType.XHR) {

        if (tabDomain !== interceptor.xhrTestDomain) {
            return interceptor._handleMissingCandidate(requestDetails.url);
        }
    }

    if (interceptor.taintedDomains[tabDomain] || (/yandex\./).test(tabDomain)) {
        return interceptor._handleMissingCandidate(requestDetails.url, true);
    }

    targetDetails = requestAnalyzer.getLocalTarget(requestDetails);
    targetPath = targetDetails.path;

    if (!targetPath) {
        return interceptor._handleMissingCandidate(requestDetails.url);
    }

    if (!files.active[targetPath]) {
        return interceptor._handleMissingCandidate(requestDetails.url);
    }

    stateManager.requests[requestDetails.requestId] = {
        tabIdentifier, targetDetails
    };

    return {
        'redirectUrl': chrome.extension.getURL(targetPath + fileGuard.secret)
    };
};

/**
 * Private Methods
 */

interceptor._handleMissingCandidate = function (requestUrl, preserveUrl) {

    let requestUrlSegments;

    if (interceptor.blockMissing === true) {

        return {
            'cancel': true
        };
    }

    if (preserveUrl === true) {

        return {
            'cancel': false
        };
    }

    requestUrlSegments = new URL(requestUrl);

    if (requestUrlSegments.protocol === Address.HTTP) {

        requestUrlSegments.protocol = Address.HTTPS;
        requestUrl = requestUrlSegments.toString();

        return {
            'redirectUrl': requestUrl
        };

    } else {

        return {
            'cancel': false
        };
    }
};

interceptor._handleStorageChanged = function (changes) {

    if (Setting.XHR_TEST_DOMAIN in changes) {
        interceptor.xhrTestDomain = changes.xhrTestDomain.newValue;
    }

    if (Setting.BLOCK_MISSING in changes) {
        interceptor.blockMissing = changes.blockMissing.newValue;
    }
};

/**
 * Initializations
 */

// Temporary list of tainted domains.
interceptor.taintedDomains = {
    '10fastfingers.com': true,
    'blog.datawrapper.de': true,
    'bundleofholding.com': true,
    'captcha.realtek.com': true,
    'cdnjs.com': true,
    'cellmapper.net': true,
    'code.world': true,
    'creativecommons.org': true,
    'de.sharkoon.com': true,
    'docs.servicenow.com': true,
    'dropbox.com': true,
    'echo-news.co.uk': true,
    'en.sharkoon.com': true,
    'epey.com': true,
    'es.sharkoon.com': true,
    'evoice.com': true,
    'fr.sharkoon.com': true,
    'freebusy.io': true,
    'gazetadopovo.com.br': true,
    'gdt.oqlf.gouv.qc.ca': true,
    'glowing-bear.org': true,
    'ico.org.uk': true,
    'identi.ca': true,
    'it.sharkoon.com': true,
    'ja.sharkoon.com': true,
    'ko.sharkoon.com': true,
    'labdoor.com': true,
    'laziska.com.pl': true,
    'lemon-aid.de': true,
    'm-ce.pl': true,
    'manualslib.com': true,
    'meslieux.paris.fr': true,
    'mgm.gov.tr': true,
    'minigames.mail.ru': true,
    'miniquadtestbench.com': true,
    'mojbytom.pl': true,
    'mojchorzow.pl': true,
    'mojegliwice.pl': true,
    'mojekatowice.pl': true,
    'mojetychy.pl': true,
    'mojmikolow.pl': true,
    'newlook.dteenergy.com': true,
    'nhm.ac.uk': true,
    'nl.sharkoon.com': true,
    'opavote.com': true,
    'openweathermap.org': true,
    'orzesze.com.pl': true,
    'piekaryslaskie.com.pl': true,
    'pl.sharkoon.com': true,
    'poedb.tw': true,
    'pt.sharkoon.com': true,
    'pyskowice.com.pl': true,
    'qwertee.com': true,
    'regentgreymouth.co.nz': true,
    'report-uri.io': true,
    'ru.sharkoon.com': true,
    'rudaslaska.com.pl': true,
    'rybnicki.com': true,
    'scan.nextcloud.com': true,
    'scotthelme.co.uk': true,
    'securityheaders.com': true,
    'securityheaders.io': true,
    'siemianowice.net.pl': true,
    'somiibo.com': true,
    'sosnowiecki.pl': true,
    'stefansundin.github.io': true,
    'swiony.pl': true,
    'timescale.com': true,
    'tr.sharkoon.com': true,
    'transcend-info.com': true,
    'udacity.com': true,
    'wodzislaw.com.pl': true,
    'yadi.sk': true,
    'yelp.com': true,
    'yourvotematters.co.uk': true,
    'zabrze.com.pl': true,
    'zh-hant.sharkoon.com': true,
    'zory.com.pl': true
};

interceptor.amountInjected = 0;
interceptor.xhrTestDomain = Address.DECENTRALEYES;
interceptor.blockMissing = false;

interceptor.relatedSettings = [];

interceptor.relatedSettings.push(Setting.AMOUNT_INJECTED);
interceptor.relatedSettings.push(Setting.XHR_TEST_DOMAIN);
interceptor.relatedSettings.push(Setting.BLOCK_MISSING);

chrome.storage.local.get(interceptor.relatedSettings, function (items) {

    interceptor.amountInjected = items.amountInjected || 0;
    interceptor.xhrTestDomain = items.xhrTestDomain || Address.DECENTRALEYES;
    interceptor.blockMissing = items.blockMissing || false;
});

/**
 * Event Handlers
 */

chrome.storage.onChanged.addListener(interceptor._handleStorageChanged);
